package org.processmining.plugins.mapvis.importdata.visualization;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.io.StringReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import net.sf.saxon.query.XQueryExpression;
import net.sf.saxon.trans.XPathException;

import org.processmining.plugins.mapvis.replay.XQueryProcessor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class MapVisMap implements Serializable {

	private static final long serialVersionUID = 1L;
	private transient Map<String, Map<AttributeKey, XQueryExpression>> queries;
	private transient Map<String, Map<AttributeKey, String>> attributeQuery;

	private transient HashMap<String, XQueryExpression> oldQueries;
	private transient HashMap<String, String> queriesAsString;

	private String name;
	private String url;
	private BufferedImage image;
	private int width;
	private int height;
	private String xAxis = null;
	private String yAxis = null;
	private boolean isOldMap = false;

	public MapVisMap(String name, String url, String xAxis, String yAxis) {
		this.name = name;
		this.url = url;
		this.xAxis = xAxis;
		this.yAxis = yAxis;

		if (yAxis != null && xAxis != null) {
			queries = new HashMap<String, Map<AttributeKey, XQueryExpression>>();
			attributeQuery = new HashMap<String, Map<AttributeKey, String>>();
		} else {
			oldQueries = new HashMap<String, XQueryExpression>();
		}
		readImage();
	}

	public MapVisMap(String name, String url) {
		this.name = name;
		this.url = url;
		oldQueries = new HashMap<String, XQueryExpression>();
		queriesAsString = new HashMap<String, String>();
		readImage();
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getUrl() {
		return url;
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}

	private void writeObject(ObjectOutputStream out) throws IOException {
		out.writeObject(name);
		out.writeObject(url);
		out.writeObject(xAxis);
		out.writeObject(yAxis);
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		ImageIO.write(image, "png", stream);
		out.writeObject(stream.toByteArray());
	}

	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
		name = (String) in.readObject();
		url = (String) in.readObject();
		xAxis = (String) in.readObject();
		yAxis = (String) in.readObject();
		byte[] bytesImage = (byte[]) in.readObject();
		image = ImageIO.read(new ByteArrayInputStream(bytesImage));
		width = image.getWidth();
		height = image.getHeight();
	}

	private synchronized void readImage() {
		try {
			image = ImageIO.read(new URL(getUrl()));
			width = image.getWidth();
			height = image.getHeight();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void addQuery(String taskName, String xquery) throws SAXException, IOException, ParserConfigurationException {
		XQueryExpression exp;
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = dbf.newDocumentBuilder();
		Document dom = builder.parse(new InputSource(new StringReader(xquery)));

		NodeList nodes = dom.getElementsByTagName("attributes");
		Element query = (Element) nodes.item(0);
		NodeList attributes = query.getElementsByTagName("attribute");

		Map<AttributeKey, String> list = new HashMap<AttributeKey, String>(attributes.getLength());

		for (int i = 0; i < attributes.getLength(); i++) {
			Element attribute = (Element) attributes.item(i);
			String x = attribute.getElementsByTagName("x").item(0).getTextContent();
			String y = attribute.getElementsByTagName("y").item(0).getTextContent();
			String coordinate = attribute.getElementsByTagName("query").item(0).getTextContent();
			AttributeKey key = new AttributeKey(x, y);
			list.put(key, coordinate);

			try {
				exp = XQueryProcessor.getInstance().getStaticContext().compileQuery(coordinate);
				if (queries.containsKey(taskName)) {
					queries.get(taskName).put(key, exp);
				} else {
					Map<AttributeKey, XQueryExpression> attrXQuery = new HashMap<AttributeKey, XQueryExpression>();
					attrXQuery.put(key, exp);
					queries.put(taskName, attrXQuery);
				}
			} catch (XPathException e) {
				e.printStackTrace();
			}
		}

		attributeQuery.put(taskName, list);
	}

	public void addImportQuery(String taskName, String xquery, String x, String y) {
		try {
			XQueryExpression exp;
			AttributeKey key = new AttributeKey(x, y);
			exp = XQueryProcessor.getInstance().getStaticContext().compileQuery(xquery);

			if (queries.containsKey(taskName)) {
				queries.get(taskName).put(key, exp);
			} else {
				Map<AttributeKey, XQueryExpression> attrXQuery = new HashMap<AttributeKey, XQueryExpression>();
				attrXQuery.put(key, exp);
				queries.put(taskName, attrXQuery);
			}

			if (attributeQuery.containsKey(taskName)) {
				attributeQuery.get(taskName).put(key, xquery);
			} else {
				Map<AttributeKey, String> list = new HashMap<AttributeKey, String>();
				list.put(key, xquery);
				attributeQuery.put(taskName, list);
			}

		} catch (XPathException e) {
			e.printStackTrace();
		}
	}

	public XQueryExpression getQuery(String name, AttributeKey attributeKey) {
		return queries.get(name).get(attributeKey);
	}

	public Set<String> getQueryNames() {
		if (queries!=null)
			return queries.keySet();
		else
			return oldQueries.keySet();
	}

	public Set<String> getOldQueryNames() {
		return oldQueries.keySet();
	}

	public Map<AttributeKey, String> getAttributeQuery(String taskName) {
		if (attributeQuery!=null)
			return attributeQuery.get(taskName);
		else
			return null;
	}

	public String getAttributeQueries(String taskName, AttributeKey attributeKey) {
		if (attributeQuery.containsKey(taskName))
			return attributeQuery.get(taskName).get(attributeKey);
		else
			return null;
	}

	public void addOldQuery(String taskName, String xquery) {
		XQueryExpression exp;
		try {
			exp = XQueryProcessor.getInstance().getStaticContext().compileQuery(xquery);
			oldQueries.put(taskName, exp);
			queriesAsString.put(taskName, xquery);
		} catch (XPathException e) {
			e.printStackTrace();
		}
	}

	public XQueryExpression getQuery(String name) {
		return oldQueries.get(name);
	}

	public boolean isTaskAvaiable(String taskName) {
		if (!attributeQuery.containsKey(taskName))
			return false;
		else
			return true;
	}

	public String getQueryString(String queryName) {
		return queriesAsString.get(queryName);
	}

	public BufferedImage getImage() {
		return image;
	}

	public String getYAxis() {
		return yAxis;
	}

	public String getXAxis() {
		return xAxis;
	}

	public boolean isOldMap() {
		return isOldMap;
	}

	public void setOldMap(boolean isOldMap) {
		this.isOldMap = isOldMap;
	}
}
