package org.processmining.plugins.mapvis.visualize;

import java.util.ArrayList;
import java.util.List;

public class AnimationTimer {

	private long modelTimeStart;
	private long modelTimeEnd;
	private long realTimeStart;
	private long realTimeEnd;
	private double speedFactor;
	private double relativePosition;
	private long updateInterval;
	private UpdateThread updateThread;

	private List<AnimationListener> listeners;

	public AnimationTimer(long modelTimeStart, long modelTimeEnd, double speedFactor, long updateInterval) {
		this.modelTimeStart = modelTimeStart;
		this.modelTimeEnd = modelTimeEnd;
		this.speedFactor = speedFactor;
		this.relativePosition = 0.0;
		this.realTimeStart = -1;
		this.realTimeEnd = -1;
		this.updateInterval = updateInterval;
		this.listeners = new ArrayList<AnimationListener>();
		this.updateThread = null;
	}

	public double getSpeedFactor() {
		return speedFactor;
	}

	public long translateModelTimeToRealTime(long modelTime) {
		double percentage = (double) (modelTime - this.modelTimeStart)
				/ (double) (this.modelTimeEnd - this.modelTimeStart);
		return realTimeStart + (long) (percentage * (realTimeEnd - realTimeStart));
	}

	public long translateModelDurationToReal(long modelDuration) {
		return (long) (modelDuration / speedFactor);
	}

	protected void updateData() {
		relativePosition = (double) (System.currentTimeMillis() - realTimeStart)
				/ (double) (realTimeEnd - realTimeStart);
		if (relativePosition > 1.0) {
			relativePosition = 1.0;
		}
	}

	protected void updateRealTimes() {
		long modelDuration = modelTimeEnd - modelTimeStart;
		long realDuration = translateModelDurationToReal(modelDuration);
		long currentTime = System.currentTimeMillis();
		this.realTimeStart = currentTime - (long) (realDuration * this.relativePosition);
		this.realTimeEnd = this.realTimeStart + realDuration;
	}

	public void setSpeedFactor(double speedFactor) {
		this.speedFactor = speedFactor;
		updateRealTimes();
	}

	public void setAbsolutePosition(long modelTime) {
		if (modelTime < modelTimeStart)
			modelTime = modelTimeStart + 1;
		if (modelTime > modelTimeEnd)
			modelTime = modelTimeEnd - 1;
		double relativePosition = (1.0D * (modelTime - modelTimeStart)) / (modelTimeEnd - modelTimeStart);
		setRelativePosition(relativePosition);
	}

	public synchronized void setRelativePosition(double relativePosition) {
		this.relativePosition = relativePosition;
		updateRealTimes();
		if (updateThread == null) {
			long modelTime = modelTimeStart + (long) ((modelTimeEnd - modelTimeStart) * relativePosition);
			for (AnimationListener listener : listeners) {
				listener.updateModelTime(modelTime);
			}
		}
	}

	public void addListener(AnimationListener animationListener) {
		listeners.add(animationListener);
	}

	public void removeAllListeners() {
		listeners.clear();
	}

	public void setUpdateInterval(long updateInterval) {
		this.updateInterval = updateInterval;
	}

	public void setUpdateFramesPerSecond(int fps) {
		this.updateInterval = 1000 / fps;
	}

	public synchronized boolean isRunning() {
		return (updateThread != null);
	}

	public synchronized void start() {
		if (updateThread == null) {
			if (relativePosition >= 1.0) {
				relativePosition = 0.0;
			}
			updateRealTimes();
			updateThread = new UpdateThread();
			updateThread.start();
			for (AnimationListener listener : listeners) {
				listener.started();
			}
		}
	}

	public synchronized void stop() {
		if (updateThread != null) {
			updateThread.stopRunning();
		}
		updateThread = null;
	}

	protected class UpdateThread extends Thread {

		protected boolean isRunning = false;
		protected long lastInvoked = -1;

		public synchronized void start() {
			if (isRunning == false) {
				isRunning = true;
				super.start();
			}
		}

		public synchronized void stopRunning() {
			isRunning = false;
			lastInvoked = -1;
			for (AnimationListener listener : listeners) {
				listener.stopped();
			}
			updateThread = null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Thread#run()
		 */
		@Override
		public void run() {
			while (isRunning) {
				updateData();
				long modelTime = modelTimeStart + (long) ((modelTimeEnd - modelTimeStart) * relativePosition);
				try {
					for (AnimationListener listener : listeners) {
						listener.updateModelTime(modelTime);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
				if (relativePosition >= 1.0) {
					stopRunning();
					return; // finished animation
				}
				// calculate sleep time
				long currentTime = System.currentTimeMillis();
				long sleepTime = updateInterval;
				if (lastInvoked > 0) {
					long cycleTime = currentTime - lastInvoked;
					if (cycleTime > updateInterval) {
						// make up for lost drawing overtime
						sleepTime -= (cycleTime - updateInterval);
						// safety check
						if (sleepTime < 0) {
							sleepTime = 0;
						}
					}
				}
				lastInvoked = currentTime;
				try {
					Thread.sleep(sleepTime);
				} catch (InterruptedException e) {
					// should be no big deal
					e.printStackTrace();
				}
			}
		}
	}

	public void removeListener(AnimationListener listener) {
		listeners.remove(listener);
	}
}
