package org.processmining.plugins.mapvis.visualize;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JPanel;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisMap;
import org.processmining.plugins.mapvis.replay.History;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema.ColoringSchema;
import org.processmining.plugins.mapvis.visualize.workitem.IndexDot;
import org.processmining.plugins.mapvis.visualize.workitem.WorkItemDot;

public class MapImagePanel extends JPanel implements ScalableComponent {
	private static final long serialVersionUID = 363108260131476112L;
	private BufferedImage image, grayImage;
	private final MapVisMap map;
	private float zoom = 1;
	private MapPanel parentPane;
	private ScalableViewPanel scalablePanel;
	private boolean mergeWithPartialOverlap = false;
	protected boolean isPainting = false;
	private boolean isBlackWhite = false;
	private PluginContext context;
	private History instHistory;
	private String relativeAttribute;
	private String relativeToAttribute;
	private Vector<IndexDot> indexDot = null;
	private long freezeTimeFrame = 0L;
	private Vector<WorkItemDot> workItemDots = new Vector<WorkItemDot>();
	private Map<Coordinate, WorkItemDot> workDots = new HashMap<Coordinate, WorkItemDot>();
	private Map<String, Map<Color, Integer>> colorData;

	public MapImagePanel(MapVisMap map, String attribute, History instHistory, PluginContext context) {
		try {
			image = map.getImage();
			this.context = context;
			this.instHistory = instHistory;
		} catch (Exception e) {
			e.printStackTrace();
		}
		this.setPreferredSize(new Dimension(image.getWidth(), image.getHeight()));
		this.setBackground(Color.BLACK);
		this.map = map;
		this.addMouseMotionListener(new MapImageRolloverListener(this));
		this.addMouseWheelListener(new MapImageRolloverListener(this));
		this.addMouseListener(new MapImageRolloverListener(this));
		AbstractWorkDotColorSchema.getInstance().setMapVisMap(map);
		AbstractWorkDotColorSchema.discriminatingAttribute.put(map, attribute);
	}

	/*
	 * find work items dots information at x and y coordinate of the current
	 * image panel
	 */
	public synchronized WorkItemDot getMergeWorkItemAt(short x, short y) {
		WorkItemDot closest = null;
		double minDistance = Integer.MAX_VALUE;
		for (WorkItemDot w : workItemDots) {
			double distance = w.distance(x, y, zoom);
			if (distance < minDistance) {
				minDistance = distance;
				closest = w;
			}
		}
		if (closest != null && minDistance <= closest.getDiameter() / (2 * zoom))
			return closest;
		else
			return null;
	}

	/*
	 * find work items dots information at x and y coordinate of the current
	 * image panel
	 */
	public synchronized WorkItemDot getWorkItemAt(short x, short y) {
		WorkItemDot closest = null;
		double minDistance = Integer.MAX_VALUE;
		for (Entry<Coordinate, WorkItemDot> wDot : workDots.entrySet()) {
			double distance = wDot.getValue().distance(x, y, zoom);
			if (distance < minDistance) {
				minDistance = distance;
				closest = wDot.getValue();
			}
		}
		if (closest != null && minDistance <= closest.getDiameter() / (2 * zoom))
			return closest;
		else
			return null;
	}

	// Convert the map panel image into a gray scale image
	public void convertToGray(BufferedImage image) {
		// create a gray scale image the same size
		grayImage = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_BYTE_GRAY);
		// Convert the original coloured image to gray scale
		ColorConvertOp op = new ColorConvertOp(image.getColorModel().getColorSpace(), grayImage.getColorModel()
				.getColorSpace(), null);
		op.filter(image, grayImage);
	}

	@Override
	public synchronized void paintComponent(Graphics g) {
		isPainting = true;
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
		// clear buffer
		g2.setColor(Color.BLACK);
		g2.fillRect(0, 0, getWidth(), getHeight());
		// set anti-aliasing
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		// apply zoom
		g2.scale(zoom, zoom);
		// draw image of the map
		g2.drawImage(image, 0, 0, null);

		//draw black and white image
		if (isBlackWhite) {
			convertToGray(image);
			g2.drawImage(grayImage, 0, 0, null);
		}
		AbstractWorkDotColorSchema.getInstance().setMapVisMap(map);

		if (parentPane.isIndextation && parentPane.isFreezeTimeIndextation()) {
			paintIndex(g2);
		} else if (mergeWithPartialOverlap) {
			paintMergeWorkItemDots(g2);
		} else {
			//draw work items dots on the map panel
			paintWorkItemDots(g2);
		}
		isPainting = false;
	}

	@SuppressWarnings("unchecked")
	public synchronized void joinWorkItemDots() {
		Vector<WorkItemDot> workItemDots2 = (Vector<WorkItemDot>) workItemDots.clone();
		boolean modified = true;
		while (modified) {
			modified = false;
			int nWorkItemDots = workItemDots2.size();
			for (int i = 0; i < nWorkItemDots; i++) {
				WorkItemDot w1 = workItemDots2.get(i);
				int j = i + 1;
				while (j < nWorkItemDots) {
					WorkItemDot w2 = workItemDots2.get(j);

					if (w1.overlapsWith(w2, zoom, mergeWithPartialOverlap)) {
						modified = true;
						WorkItemDot merged = w1.joinTo(w2, true);
						workItemDots2.remove(j);
						nWorkItemDots--;
						workItemDots2.set(i, merged);
						w1 = merged;
					} else {
						j++;
					}
				}
			}
		}
		workItemDots = workItemDots2;
	}

	/*
	 * get work item dot of freeze time frame
	 */
	public void findDiffBetweenFrame(Map<Coordinate, WorkItemDot> myCurrentdots,
			Map<Coordinate, WorkItemDot> freezedot, long modellingTime) {
		indexDot = new Vector<IndexDot>();

		if (modellingTime != freezeTimeFrame) {
			Map<Coordinate, WorkItemDot> tempFreezeDot = new HashMap<Coordinate, WorkItemDot>();
			tempFreezeDot.putAll(freezedot);
			for (Entry<Coordinate, WorkItemDot> dot : myCurrentdots.entrySet()) {
				WorkItemDot wDot = tempFreezeDot.get(dot.getKey());
				if (wDot == null) {
					IndexDot index = new IndexDot(dot.getKey(), dot.getValue().getMultiWorkDotSize());
					indexDot.add(index);
				} else {
					int size = dot.getValue().getMultiWorkDotSize() - wDot.getMultiWorkDotSize();
					if (Math.abs(size) > 0) {
						IndexDot index = new IndexDot(dot.getKey(), size);
						indexDot.add(index);
						tempFreezeDot.remove(dot.getKey());
					}
				}
			}
			for (Entry<Coordinate, WorkItemDot> dot : tempFreezeDot.entrySet()) {
				WorkItemDot wDot = myCurrentdots.get(dot.getKey());
				if (wDot == null) {
					IndexDot index = new IndexDot(dot.getKey(), -dot.getValue().getMultiWorkDotSize());
					indexDot.add(index);
				}
			}
		}
	}

	/*
	 * for painting positive and negative dot from the freeze time frame
	 */
	private synchronized void paintIndex(Graphics2D g2) {
		for (IndexDot index : indexDot)
			index.paint(g2, zoom);
	}

	/*
	 * Initialise the workItem Dot when merge dot option is ON, on the image of
	 * the map panel
	 */
	private synchronized void paintMergeWorkItemDots(Graphics2D g2D) {
		colorData = new TreeMap<String, Map<Color, Integer>>();
		ColoringSchema schema = AbstractWorkDotColorSchema.schemaMap.get(map);

		for (WorkItemDot wDot : workItemDots) {
			if (parentPane.isRelativeToAttributeOn) {
				wDot.calPercentage(relativeAttribute, relativeToAttribute);
			}
			wDot.paint(g2D, zoom, parentPane.isRelativeToAttributeOn);
			if (schema != null) {
				if (schema.toString().equalsIgnoreCase("AGEDONACTIVITYMEANTIME")) {
					colorData.putAll(wDot.colorData());
				}
			}
		}

		if (schema != null) {
			if (schema.toString().equalsIgnoreCase("AGEDONACTIVITYMEANTIME")) {
				parentPane.legend.getAgedata(colorData);
				parentPane.legend.recompute();
			}
		}
	}

	/*
	 * Initialise the workItem Dot on the image of the map panel
	 */
	private synchronized void paintWorkItemDots(Graphics2D g2D) {
		colorData = new TreeMap<String, Map<Color, Integer>>();
		ColoringSchema schema = AbstractWorkDotColorSchema.schemaMap.get(map);
		for (Entry<Coordinate, WorkItemDot> wDot : workDots.entrySet()) {
			if (parentPane.isRelativeToAttributeOn) {
				wDot.getValue().calPercentage(relativeAttribute, relativeToAttribute);
			}
			wDot.getValue().paint(g2D, zoom, parentPane.isRelativeToAttributeOn);

			if (schema != null) {
				if (schema.toString().equalsIgnoreCase("AGEDONACTIVITYMEANTIME")) {
					colorData.putAll(wDot.getValue().colorData());
				}
			}
		}

		if (schema != null) {
			if (schema.toString().equalsIgnoreCase("AGEDONACTIVITYMEANTIME")) {
				parentPane.legend.getAgedata(colorData);
				parentPane.legend.recompute();
			}
		}
	}

	public Map<String, Map<Color, Integer>> getColordata() {
		return this.colorData;
	}

	public void setZoom(float zoom) {
		this.zoom = zoom;
		this.setPreferredSize(new Dimension(Math.round(image.getWidth() * zoom), Math.round(image.getHeight() * zoom)));
		parentPane.recalc();
	}

	public void createSubLog(ArrayList<String> workDots) {
		ExportSubLog export = new ExportSubLog(instHistory, context);
		export.publishSubLog(workDots);
	}

	public void setParentPane(MapPanel parentPane) {
		this.parentPane = parentPane;
	}

	public MapPanel getParentPane() {
		return parentPane;
	}

	public synchronized void clearMergeWorkItemDots() {
		workItemDots.clear();
	}

	public synchronized void setMergeWorkItemDots(List<WorkItemDot> wDot) {
		workItemDots.addAll(wDot);
	}

	public synchronized void clearWorkItemDots() {
		workDots.clear();
	}

	public synchronized void setWorkItemDots(Map<Coordinate, WorkItemDot> wDot) {
		workDots.putAll(wDot);
	}

	public JComponent getComponent() {
		return this;
	}

	public double getScale() {
		return zoom;
	}

	public void setScale(double newScale) {
		this.setZoom((float) newScale);
	}

	public void setScalablePanel(ScalableViewPanel scalablePanel) {
		this.scalablePanel = scalablePanel;
	}

	public ScalableViewPanel getScalablePanel() {
		return scalablePanel;
	}

	public boolean isMergeWithPartialOverlap() {
		return mergeWithPartialOverlap;
	}

	public void setMergeWithPartialOverlap(boolean mergeOverllapingDots) {
		this.mergeWithPartialOverlap = mergeOverllapingDots;
	}

	public MapVisMap getMapVisMap() {
		return map;
	}

	public void recomputeMapColor() {
		setBlackWhite(parentPane.isBlackNWhite);
	}

	public void setBlackWhite(boolean BlackNWhite) {
		isBlackWhite = BlackNWhite;
	}

	public long getFreezeTimeFrame() {
		return freezeTimeFrame;
	}

	public void setFreezeTimeFrame(long freezeTimeFrame) {
		this.freezeTimeFrame = freezeTimeFrame;
	}

	public void getRelativeAttribute(String attribute) {
		this.relativeAttribute = attribute;
	}

	public void getRelativeToAttribute(String attribute) {
		this.relativeToAttribute = attribute;
	}

	public double getHeightInView() {
		return this.getPreferredSize().getHeight();
	}

	public double getWidthInView() {
		return this.getPreferredSize().getWidth();
	}

	public void removeUpdateListener(UpdateListener listener) {
	}

	public void addUpdateListener(UpdateListener listener) {
	}
}