package org.processmining.plugins.mapvis.visualize;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.RoundRectangle2D;

import javax.swing.JComponent;

import org.processmining.plugins.mapvis.VisData;
import org.processmining.plugins.mapvis.replay.History;
import org.processmining.plugins.mapvis.replay.TimeStampArray;


public class SeekPane extends JComponent implements AnimationListener, MouseMotionListener, MouseListener {

	private static final long serialVersionUID = 6046298110063068563L;
	private Color waveColor = new Color(200, 0, 0);
	
	private boolean timerRunningBeforeSeek = false;
	private boolean isSeeking = false;
	private boolean justStarted = true;
	private int seekOffset = 0;
	
	private AnimationTimer timer;
	private long start;
	private long end;
	private long duration;
	private float position;
	
	private float[] activity;
	private int[] waveX;
	private int[] waveY;
	
	private int height = 40;
	private int hBorder = 7;
	private int topBorder = 10;
	private int bottomBorder = 4;
	private History instHistory;
	
	public SeekPane(VisData visData, AnimationTimer timer) {
		this.setDoubleBuffered(true);
		this.instHistory=visData.getHistory();
		this.timer = timer;
		this.start = visData.getStart();
		this.end = visData.getEnd();
		this.duration = end - start;
		this.position = 0.0f;
		this.setMinimumSize(new Dimension(200, height));
		this.setMaximumSize(new Dimension(2000, height));
		this.setPreferredSize(new Dimension(700, height));
		measureActivity();
		timer.addListener(this);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
	}

	public void updateModelTime(long modelTime) {
		this.position = (float)(modelTime - this.start) / (float)duration;
		//History history = instHistory;
		TimeStampArray.getInstance().updateModelingTime(modelTime);
		
		if (this.isSeeking || this.justStarted) {
			TimeStampArray.getInstance().seekCounter(modelTime);
			this.justStarted = false;
		} else {
			TimeStampArray.getInstance().updateCounter(modelTime);
		}
		
		repaint();
	}

	public void started() {
		this.justStarted = true;
	}
		public void stopped() {
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		int width = getWidth();
		int height = getHeight();
		RoundRectangle2D.Float clip = new RoundRectangle2D.Float(0f, 0f, width, height, 10f, 10f);
		g2d.setColor(new Color(0, 0, 0));
		g2d.fill(clip);
		g2d.setClip(clip);
		// paint wave
		int wavePoints = activity.length;
		int waveHeight = height - topBorder - bottomBorder;
		float maxOffset = (float)waveHeight / 2;
		int waveCenter = topBorder + (waveHeight / 2);
		int waveWidth = width - hBorder - hBorder;
		waveX[0] = hBorder;
		waveY[0] = waveCenter;
		waveX[wavePoints + 1] = width - hBorder;
		waveY[wavePoints + 1] = waveCenter;
		float step = (float)waveWidth / (float)wavePoints;
		float startX = hBorder + (step / 2f);
		int x, offset;
		int index = 1;
		int counterIndex = waveX.length - 1;
		for(int i=0; i<wavePoints; i++) {
			x = (int)(startX + (step * i));
			offset = (int)(maxOffset * activity[i]);
			waveX[index] = x;
			waveY[index] = waveCenter - offset - 1;
			waveX[counterIndex] = x;
			waveY[counterIndex] = waveCenter + offset;
			index++;
			counterIndex--;
		}
		g2d.setColor(waveColor);
		g2d.fillPolygon(waveX, waveY, waveX.length);
		// draw reflection
		GradientPaint gradient;
		gradient = new GradientPaint(0, 0, new Color(0, 0, 0, 0), 
				0, height, new Color(0, 0, 0, 60), false);
		g2d.setPaint(gradient);
		g2d.fillRect(0, 0, width + 1, height + 1);
		gradient = new GradientPaint(0, 0, new Color(200, 200, 200, 10), 
				0, waveCenter, new Color(200, 200, 200, 50), false);
		g2d.setPaint(gradient);
		g2d.fillRect(0, 0, width + 1, waveCenter);
		gradient = new GradientPaint(0, waveCenter, new Color(200, 200, 200, 50), 
				0, waveCenter + 3, new Color(200, 200, 200, 0), false);
		g2d.setPaint(gradient);
		g2d.fillRect(0, waveCenter, width + 1, 3);
		// draw handle / progress indicator
		int handleCenter = hBorder + (int)(this.position * waveWidth);
		int[] handleX = new int[] {handleCenter + 5, handleCenter + 5, handleCenter, handleCenter - 5, handleCenter - 5};
		int[] handleY = new int[] {0, waveCenter + 5, waveCenter + 10, waveCenter + 5, 0};
		gradient = new GradientPaint(0, 0, new Color(200, 200, 200, 0), 
				0, 5, new Color(200, 200, 200, 110), false);
		g2d.setPaint(gradient);
		g2d.fillPolygon(handleX, handleY, 5);
		gradient = new GradientPaint(0, 0, new Color(200, 200, 200, 0), 
				0, 5, new Color(220, 220, 220, 120), false);
		g2d.setPaint(gradient);
		g2d.drawPolyline(handleX, handleY, 5);
		g2d.setColor(new Color(20, 0, 0, 180));
		g2d.drawLine(handleCenter, 0, handleCenter, waveCenter + 2);
		// paint border
		g2d.setColor(new Color(0, 0, 0, 220));
		g2d.draw(clip);
	}

	protected void measureActivity() {
		activity = new float[instHistory.getNumStates()];
		waveX = new int[(activity.length * 2) + 2];
		waveY = new int[waveX.length];
		float maxActivity = 0.0f;
		for(int i=0; i<activity.length; i++) {
			//activity[i] = this.visData.getHistory().getNumberActivity(i);
			activity[i] = TimeStampArray.getInstance().getNumberActivity(i);
			if(activity[i] > maxActivity) {
				maxActivity = activity[i];
			}
		}
		// normalize
		for(int i=0; i<activity.length; i++) {
			activity[i] = ((activity[i]*activity[i])/(maxActivity*maxActivity));
		}
	}

	public void mouseDragged(MouseEvent e) {
		if(this.isSeeking) {
			float waveWidth = this.getWidth() - hBorder - hBorder;
			int mouseX = e.getX() + this.seekOffset;
			float percentage = (mouseX - hBorder) / waveWidth;
			if(percentage < 0f) {
				percentage = 0f;
			} else if(percentage > 1f) {
				percentage = 1f;
			}
			this.position = percentage;
			this.timer.setRelativePosition(percentage);
			repaint();
		}
	}

	public void mouseMoved(MouseEvent e) {
		if(this.isSeeking) {
			float waveWidth = this.getWidth() - hBorder - hBorder;
			int mouseX = e.getX() + this.seekOffset;
			float percentage = (mouseX - hBorder) / waveWidth;
			if(percentage < 0f) {
				percentage = 0f;
			} else if(percentage > 1f) {
				percentage = 1f;
			}
			this.position = percentage;
			this.timer.setRelativePosition(percentage);
			repaint();
		}
	}

	public void mousePressed(MouseEvent e) {
		int mouseX = e.getX();
		int handleCenter = hBorder + (int)(this.position * (this.getWidth() - hBorder - hBorder));
		if(mouseX >= (handleCenter - 5) && mouseX <= (handleCenter + 5)) {
			this.isSeeking = true;
			this.seekOffset = handleCenter - mouseX;
			this.timerRunningBeforeSeek = this.timer.isRunning();
			this.timer.stop();
		}
	}


	public void mouseReleased(MouseEvent e) {
		if(this.isSeeking) {
			float waveWidth = this.getWidth() - hBorder - hBorder;
			int mouseX = e.getX() + this.seekOffset;
			float percentage = (mouseX - hBorder) / waveWidth;
			if(percentage < 0f) {
				percentage = 0f;
			} else if(percentage > 1f) {
				percentage = 1f;
			}
			this.timer.setRelativePosition(percentage);
			this.isSeeking = false;
			if(this.timerRunningBeforeSeek) {
				this.timer.start();
			}
		}
	}
	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	
	

}
