package org.processmining.plugins.mapvis.visualize;

import java.awt.BasicStroke;
import java.awt.Color;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ViewInteractionPanel;
import org.processmining.plugins.mapvis.replay.TimeStampArray;

public class TimeChartPanel implements ViewInteractionPanel, AnimationListener {

	private final ChartPanel chartPanel;
	private boolean visible;
	ValueMarker originalEnd;
	private final String panelName;

	public TimeChartPanel(String panelName, TimeSeriesCollection dataset, AnimationTimer timer, Color[] seriesColors,
			boolean legend, Boolean showNumbers) {

		this.panelName = panelName;
		JFreeChart chart = ChartFactory.createTimeSeriesChart("", // title
				"Date", // x-axis label
				"Number of dots", // y-axis label
				dataset, // data
				legend, // create legend
				true, // generate tooltips
				false // generate URLs
				);

		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setDomainGridlinesVisible(true);
		plot.setRangeGridlinesVisible(true);
		plot.setRangeGridlinePaint(Color.WHITE);
		plot.setDomainGridlinePaint(Color.WHITE);
		plot.setDomainAxis(new DateAxis("", TimeZone.getDefault(), Locale.ENGLISH));
		if (!showNumbers) {
			ValueAxis range = plot.getRangeAxis();
			range.setVisible(false);
		}

		XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		for (int i = 0; i < seriesColors.length; i++) {
			renderer.setSeriesLinesVisible(i, true);
			renderer.setSeriesShapesVisible(i, false);
			renderer.setSeriesPaint(i, seriesColors[i]);
		}
		for (int ch = 0; ch < dataset.getSeriesCount(); ch++)
			renderer.setSeriesToolTipGenerator(ch, new StandardXYToolTipGenerator() {
				private static final long serialVersionUID = 1L;

				public String generateToolTip(XYDataset dataSet, int series, int item) {
					double yValue = dataSet.getYValue(series, item);
					double xValue = dataSet.getXValue(series, item);
					return new Date((long) xValue) + ": " + yValue + " dots";
				}
			});

		plot.setRenderer(renderer);
		plot.setBackgroundPaint(Color.white);
		
		try {
			if (timer != null) {
				originalEnd = new ValueMarker(dataset.getStartXValue(0, 0), Color.RED, new BasicStroke(10));
				plot.addDomainMarker(originalEnd);
			}
		} catch (IndexOutOfBoundsException e) {
		//	System.out.println(e);
			
		}
		

		chartPanel = new ChartPanel(chart);
		chartPanel.setMouseZoomable(true, false);
		chartPanel.getChart().setBackgroundPaint(new Color(0, 0, 0, 0)); // transparent black
		chart.getPlot().setBackgroundPaint(Color.BLACK);

		if (timer != null)
			chartPanel.addChartMouseListener(new MyChartMouseListener(chartPanel, timer));
	}

	public void updateModelTime(final long modelTime) {
		if (!visible)
			return;

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					originalEnd.setValue(modelTime);
				} catch (Exception e) {
					//System.out.println(e);
				}
				
			}
		});
	}

	public void setLabelPosChart(boolean showNumbers) {
		XYPlot plot = (XYPlot) chartPanel.getChart().getPlot();
		plot.getRangeAxis().setVisible(showNumbers);
	}

	public JComponent getComponent() {
		return chartPanel;
	}

	public double getHeightInView() {
		return 300;
	}

	public String getPanelName() {
		return panelName;
	}

	public double getWidthInView() {
		return 750;
	}

	public void willChangeVisibility(boolean visible) {
		this.visible = visible;
		if (visible)
			updateModelTime(TimeStampArray.getInstance().getModelingTime());
	}

	public void setParent(ScalableViewPanel viewPanel) {
	}

	public void setScalableComponent(ScalableComponent scalable) {
	}

	public void updated() {

	}

	public void started() {

	}

	public void stopped() {

	}

}
