package org.processmining.plugins.mapvis.visualize.workitem;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;

import org.mapdb.Serializer;
import org.processmining.plugins.mapvis.replay.ActivityKey;
import org.processmining.plugins.mapvis.replay.NameLookUp;

/**
 * Custom {@link Serializer} for {@link AbstractWorkItemDot}. For each
 * AbstractWorkItemDot take up the following amount of disk-space:
 * <p>
 * <li>4 bytes for their ID,
 * <li>2 byte for their type of workItem dot,
 * <li>2 byte for their state of workItem dot,
 * <li>8 bytes creation time,
 * <li>4 bytes for event case-Id,if it is a workItem dot
 * <li>x bytes for event name,if their it is a workItem dot
 * <li>2 bytes for x-coordinate value,if it is a workItem dot
 * <li>2 bytes for y-coordinate value,if it is a workItem dot
 * 
 * @author n.gupta
 * 
 */

public class AbstractWorkItemDotSerializer extends   Serializer<ArrayList<AbstractWorkItemDot>> implements Serializable {

	private static final long serialVersionUID = 1L;
	private static byte INVALID_WORK_ITEM = 0;
	private static byte WORKITEM_WORK_ITEM = 1;
	private static byte HIDDEN_WORK_ITEM = 2;

	public AbstractWorkItemDotSerializer() {
		super();
	}

	 @Override
	public void serialize(DataOutput out, ArrayList<AbstractWorkItemDot> value) throws IOException {
		out.writeInt(value.size()); // 4 bytes
		for (AbstractWorkItemDot dot : value) {
			if (dot instanceof WorkItemDot) { // 
				out.writeByte(WORKITEM_WORK_ITEM);
				out.writeByte(dot.getState());
				out.writeLong(dot.getCreationTime());
				out.writeInt(dot.getKey().getValueForCaseId());		
				short activityValue =NameLookUp.getInstance().getValueByActivityName(dot.getKey().getActivityName());
				out.writeShort(activityValue);
				out.writeShort(((WorkItemDot) dot).getX());
				out.writeShort(((WorkItemDot) dot).getY());
			} else if (dot instanceof HiddenWorkItemDot) { // 10 bytes
				out.writeByte(HIDDEN_WORK_ITEM);
				out.writeByte(dot.getState());
				out.writeLong(dot.getCreationTime());
				out.writeInt(dot.getKey().getValueForCaseId());
				short activityValue =NameLookUp.getInstance().getValueByActivityName(dot.getKey().getActivityName());
				out.writeShort(activityValue);
			} else if (dot instanceof InvalidWorkItemDot) { // 10 bytes
				out.writeByte(INVALID_WORK_ITEM);
				out.writeByte(dot.getState());
				out.writeLong(dot.getCreationTime());
				out.writeInt(dot.getKey().getValueForCaseId());
				short activityValue =NameLookUp.getInstance().getValueByActivityName(dot.getKey().getActivityName());
				out.writeShort(activityValue);
			} else {
				throw new RuntimeException("Unkown work item dot");
			}
		}
	}

	 @Override
	public ArrayList<AbstractWorkItemDot> deserialize(DataInput in, int available) throws IOException {
		int size = in.readInt();
		ArrayList<AbstractWorkItemDot> list = new ArrayList<>(size);
		for (int i = 0; i < size; i++) {
			byte type = in.readByte();
			if (type == INVALID_WORK_ITEM) {
				byte state = in.readByte();
				long creationTime = in.readLong();
				String caseIdValue = NameLookUp.getInstance().getCaseIdbyValue(in.readInt());
				String activityName = NameLookUp.getInstance().getActivityNamebyValue(in.readShort());
				ActivityKey key = new ActivityKey(activityName, caseIdValue);
				list.add(new InvalidWorkItemDot(state, creationTime,key));
			} else if (type == WORKITEM_WORK_ITEM) {
				byte state = in.readByte();
				long creationTime = in.readLong();
				String caseIdValue = NameLookUp.getInstance().getCaseIdbyValue(in.readInt());
				String activityName = NameLookUp.getInstance().getActivityNamebyValue(in.readShort());
				ActivityKey key = new ActivityKey(activityName, caseIdValue);
				short x = in.readShort();
				short y = in.readShort();
				list.add(new WorkItemDot(x, y, state, creationTime,key));
			} else if (type == HIDDEN_WORK_ITEM) {
				byte state = in.readByte();
				long creationTime = in.readLong();
				String caseIdValue = NameLookUp.getInstance().getCaseIdbyValue(in.readInt());
				String activityName = NameLookUp.getInstance().getActivityNamebyValue(in.readShort());
				ActivityKey key = new ActivityKey(activityName, caseIdValue);
				list.add(new HiddenWorkItemDot(state, creationTime,key));
			} else {
				throw new RuntimeException("Unkown work item dot");
			}
		}
		return list;
	}

	 @Override
	public int fixedSize() {
		return -1;
	}

}
